#ifndef FrameCPP_VERSION_3_FrameH_HH
#define FrameCPP_VERSION_3_FrameH_HH

#include "general/types.hh"

#include "framecpp/Version3/FrameSpec.hh"

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/FrameH.hh"

#include "framecpp/Version3/GPSTime.hh"
#include "framecpp/Version3/STRING.hh"
#include "framecpp/Version3/PTR_STRUCT.hh"
#include "framecpp/Version3/FrSE.hh"

#include "framecpp/Version3/FrDetector.hh"
#include "framecpp/Version3/FrHistory.hh"
#include "framecpp/Version3/FrProcData.hh"
#include "framecpp/Version3/FrRawData.hh"
#include "framecpp/Version3/FrSimData.hh"
#include "framecpp/Version3/FrSummary.hh"
#include "framecpp/Version3/FrTrigData.hh"
#include "framecpp/Version3/FrVect.hh"

namespace FrameCPP
{
  namespace Version_3
  {
    //===================================================================
    //===================================================================
    class FrameHStorage
    {
    public:
      typedef INT_4U run_type;

      /// \brief Default constructor
      FrameHStorage( );

      /// \brief Copy Constructor
      FrameHStorage( const FrameHStorage& Source );

      /// \brief Constructor
      FrameHStorage( const std::string& name,
		     run_type run,
		     INT_4U frame,
		     const GPSTime& time,
		     INT_2U uLeapS,
		     INT_4S LocalTime,
		     const REAL_8 dt,
		     INT_4U dqual );

      const std::string& GetName( ) const;
  
      run_type GetRun( ) const;
  
      INT_4U GetFrame( ) const;
  
      const GPSTime& GetGTime( ) const;
  
      INT_2U GetULeapS( ) const;
  
      REAL_8 GetDt( ) const;

      INT_4S GetLocalTime( ) const;

      void SetRun( run_type Run );

      void SetDt( REAL_8 Dt );
  
      void SetGTime( const GPSTime& GTime );

    protected:

      static void dataDescription( Common::Description& Desc );

      struct data_type
      {
	STRING	name;
	run_type	run;
	INT_4U	frame;
	GPSTime	GTime;
	INT_2U	ULeapS;
	INT_4S  localTime;
	REAL_8	dt;

	inline
	data_type( )
	  : dt( 0 )
	{
	}

	inline
	data_type( const data_type& Source )
	  :   name( Source.name ),
	      run( Source.run ),
	      frame( Source.frame ),
	      GTime( Source.GTime ),
	      ULeapS( Source.ULeapS ),
	      dt( Source.dt )
	{
	}

	inline
	data_type( const std::string& Name,
		   run_type Run,
		   INT_4U Frame,
		   const GPSTime& GTime,
		   INT_2U ULeapS,
		   INT_4U LocalTime,
		   const REAL_8& DT,
		   INT_4U DQual )
	  : name( Name ),
	    run( Run ),
	    frame( Frame ),
	    GTime( GTime ),
	    ULeapS( ULeapS ),
	    localTime( LocalTime ),
	    dt( DT )
	{
	}

	FrameCPP::cmn_streamsize_type Bytes( ) const;

	inline bool
	operator==( const data_type& RHS ) const
	{
	  return ( ( this == &RHS )
		   || ( ( name == RHS.name )
			&& ( run == RHS.run )
			&& ( frame == RHS.frame )
			&& ( GTime == RHS.GTime )
			&& ( ULeapS == RHS.ULeapS )
			&& ( localTime == RHS.localTime )
			&& ( dt == RHS.dt )
			) );
	}

	inline void
	operator( )( Common::IStream& Stream )
	{
	  Stream
	    >> name
	    >> run
	    >> frame
	    >> GTime
	    >> ULeapS
	    >> localTime
	    >> dt
	    ;
	}

	inline void
	operator( )( Common::OStream& Stream ) const
	{
	  Stream
	    << name
	    << run
	    << frame
	    << GTime
	    << ULeapS
	    << localTime
	    << dt
	    ;
	}

      };

      data_type	m_data;
    };

    inline FrameCPP::cmn_streamsize_type FrameHStorage::data_type::
    Bytes( ) const
    {
      return
	name.Bytes( )
	+ sizeof( run )		// run;
	+ sizeof( frame )	// frame;
	+ sizeof( INT_4U )	// GTime Seconds;
	+ sizeof( INT_4U )	// GTime Nanoeconds;
	+ sizeof( ULeapS )	// ULeapS;
	+ sizeof( localTime )
	+ sizeof( dt )		// dt;
	;
    }

    inline const std::string& FrameHStorage::
    GetName( ) const
    {
      return m_data.name;
    }
  
    inline FrameHStorage::run_type FrameHStorage::
    GetRun( ) const
    {
      return m_data.run;
    }
  
    inline INT_4U FrameHStorage::
    GetFrame( ) const
    {
      return m_data.frame;
    }
  
    inline const GPSTime& FrameHStorage::
    GetGTime( ) const
    {
      return m_data.GTime;
    }
  
    inline INT_2U FrameHStorage::
    GetULeapS( ) const
    {
      return m_data.ULeapS;
    }
  
    inline REAL_8 FrameHStorage::
    GetDt( ) const
    {
      return m_data.dt;
    }
  
    inline INT_4S FrameHStorage::
    GetLocalTime( ) const
    {
      return m_data.localTime;
    }
  
    inline void FrameHStorage::
    SetRun( run_type Run )
    {
      m_data.run = Run;
    }

    inline void FrameHStorage::
    SetDt( REAL_8 Dt )
    {
      m_data.dt = Dt;
    }
  
    inline void FrameHStorage::
    SetGTime( const GPSTime& GTime )
    {
      m_data.GTime = GTime;
    }   
   
    inline void FrameHStorage::
    dataDescription( Common::Description& Desc )
    {
      Desc( FrSE( "name", "STRING",
		  "Name of project or other experiment description") );
      Desc( FrSE( "run", "INT_4U",
		  "Run number(number<0 reserved for simulated data);"
		  " monotonic for experimental runs.") );
      Desc( FrSE( "frame", "INT_4U",
		  "Frame number, monotonically increasing until end of run,"
		  " re-starting from 0 with each new run") );
      Desc( FrSE( "GTimeS", "INT_4U",
		  "Frame start time in GPS seconds") );
      Desc( FrSE( "GTimeN", "INT_4U",
		  "Frame start time residual, integer nanoseconds") );
      Desc( FrSE( "ULeapS", "INT_2U",
		  "The integer number of leap seconds between GPS/TAI and UTC"
		  " in the epoch when the frame is written") );
      Desc( FrSE( "localTime", "INT_4S",
		  "Local seasonal time - UTC") );
      Desc( FrSE( "dt", "REAL_8",
		  "Frame length in seconds.") );

    }

    //===================================================================
    //===================================================================
    template< typename TDetector, typename TTrigData, typename THistory,
	      typename TProcData, typename TRawData, typename TSimData,
	      typename TSummary, typename TVect >
    class FrameHRefsT
    {
    public:
      static const INT_2U MAX_REF = 12;

      typedef TRawData rawData_type;

      typedef typename Common::SearchContainer< TVect, &TVect::GetName >
      type_type;
      typedef typename  type_type::iterator type_iterator;
      typedef typename  type_type::const_iterator const_type_iterator;
      typedef typename  type_type::hash_iterator type_hash_iterator;
      typedef typename  type_type::const_hash_iterator const_type_hash_iterator;

      typedef typename Common::SearchContainer< TVect, &TVect::GetName  >
      user_type;
      typedef typename  user_type::iterator user_iterator;
      typedef typename  user_type::const_iterator const_user_iterator;
      typedef typename  user_type::hash_iterator user_hash_iterator;
      typedef typename  user_type::const_hash_iterator const_user_hash_iterator;

      typedef typename Common::SearchContainer< TDetector, &TDetector::GetName >
      detectSim_type;
      typedef typename  detectSim_type::iterator detectSim_iterator;
      typedef typename  detectSim_type::const_iterator const_detectSim_iterator;
      typedef typename  detectSim_type::hash_iterator detectSim_hash_iterator;
      typedef typename  detectSim_type::const_hash_iterator const_detectSim_hash_iterator;

      typedef typename Common::SearchContainer< TDetector, &TDetector::GetName >
      detectProc_type;
      typedef typename  detectProc_type::iterator detectProc_iterator;
      typedef typename  detectProc_type::const_iterator const_detectProc_iterator;
      typedef typename  detectProc_type::hash_iterator detectProc_hash_iterator;
      typedef typename  detectProc_type::const_hash_iterator const_detectProc_hash_iterator;

      typedef typename Common::SearchContainer< TTrigData, &TTrigData::GetNameSlow >
      trigData_type;
      typedef typename  trigData_type::iterator trigData_iterator;
      typedef typename  trigData_type::const_iterator const_trigData_iterator;
      typedef typename  trigData_type::hash_iterator trigData_hash_iterator;
      typedef typename  trigData_type::const_hash_iterator const_trigData_hash_iterator;

      typedef typename Common::SearchContainer< THistory, &THistory::GetName >
      history_type;
      typedef typename  history_type::iterator history_iterator;
      typedef typename  history_type::const_iterator const_history_iterator;
      typedef typename  history_type::hash_iterator history_hash_iterator;
      typedef typename  history_type::const_hash_iterator const_history_hash_iterator;

      typedef typename Common::SearchContainer< TProcData, &TProcData::GetNameSlow >
      procData_type;
      typedef typename  procData_type::iterator procData_iterator;
      typedef typename  procData_type::const_iterator const_procData_iterator;
      typedef typename  procData_type::hash_iterator procData_hash_iterator;
      typedef typename  procData_type::const_hash_iterator const_procData_hash_iterator;

      typedef typename Common::SearchContainer< TProcData, &TProcData::GetNameSlow >
      strain_type;
      typedef typename  strain_type::iterator strain_iterator;
      typedef typename  strain_type::const_iterator const_strain_iterator;
      typedef typename  strain_type::hash_iterator strain_hash_iterator;
      typedef typename  strain_type::const_hash_iterator const_strain_hash_iterator;

      typedef typename Common::SearchContainer< TSimData, &TSimData::GetNameSlow >
      simData_type;
      typedef typename  simData_type::iterator simData_iterator;
      typedef typename  simData_type::const_iterator const_simData_iterator;
      typedef typename  simData_type::hash_iterator simData_hash_iterator;
      typedef typename  simData_type::const_hash_iterator const_simData_hash_iterator;

      typedef typename Common::SearchContainer< TSummary, &TSummary::GetName >
      summaryData_type;
      typedef typename  summaryData_type::iterator summaryData_iterator;
      typedef typename  summaryData_type::const_iterator const_summaryData_iterator;
      typedef typename  summaryData_type::hash_iterator summaryData_hash_iterator;
      typedef typename  summaryData_type::const_hash_iterator const_summaryData_hash_iterator;

      typedef typename Common::SearchContainer< TVect, &TVect::GetName >
      auxData_type;
      typedef typename  auxData_type::iterator auxData_iterator;
      typedef typename  auxData_type::const_iterator const_auxData_iterator;
      typedef typename  auxData_type::hash_iterator auxData_hash_iterator;
      typedef typename  auxData_type::const_hash_iterator const_auxData_hash_iterator;

      inline FrameHRefsT( )
      {
      }

      inline FrameHRefsT( const FrameHRefsT& Source,
			  const Common::FrameH::detector_copy_type Copy
			  = Common::FrameH::DEEP_COPY_DETECTORS )
	: m_refs( Source.m_refs, Copy )
      {
      }

      inline const rawData_type* GetRawData( ) const
      {
        return m_refs.rawData;
      }
  
      inline rawData_type* GetRawData( )
      {
        return m_refs.rawData;
      }
  
      inline const type_type& RefType( ) const
      {
        return m_refs.type;
      }
  
      inline type_type& RefType( )
      {
        return m_refs.type;
      }
  
      inline const user_type& RefUser( ) const
      {
        return m_refs.user;
      }
  
      inline user_type& RefUser( )
      {
        return m_refs.user;
      }
  
      inline const detectSim_type& RefDetectSim( ) const
      {
        return m_refs.detectSim;
      }
  
      inline detectSim_type& RefDetectSim( )
      {
        return m_refs.detectSim;
      }
  
      inline const detectProc_type& RefDetectProc( ) const
      {
        return m_refs.detectProc;
      }
  
      inline detectProc_type& RefDetectProc( )
      {
        return m_refs.detectProc;
      }
  
      inline const history_type& RefHistory( ) const
      {
        return m_refs.history;
      }
  
      inline history_type& RefHistory( )
      {
        return m_refs.history;
      }
  
      inline const procData_type& RefProcData( ) const
      {
        return m_refs.procData;
      }
  
      inline procData_type& RefProcData( )
      {
        return m_refs.procData;
      }
  
      inline const procData_type& RefStrain( ) const
      {
        return m_refs.strain;
      }
  
      inline procData_type& RefStrain( )
      {
        return m_refs.strain;
      }
  
      inline const simData_type& RefSimData( ) const
      {
        return m_refs.simData;
      }
  
      inline simData_type& RefSimData( )
      {
        return m_refs.simData;
      }
  
      inline const trigData_type& RefTrigData( ) const
      {
        return m_refs.trigData;
      }
  
      inline trigData_type& RefTrigData( )
      {
        return m_refs.trigData;
      }

      inline const summaryData_type& RefSummaryData( ) const
      {
        return m_refs.summaryData;
      }
  
      inline summaryData_type& RefSummaryData( )
      {
        return m_refs.summaryData;
      }
  
      inline const auxData_type& RefAuxData( ) const
      {
        return m_refs.auxData;
      }
  
      inline auxData_type& RefAuxData( )
      {
        return m_refs.auxData;
      }
  
      inline void SetRawData( const rawData_type& r )
      {
	if ( m_refs.m_owns_raw_data )
	{
	  delete m_refs.rawData;
	  m_refs.rawData = (rawData_type*)NULL;
	}
    
	m_refs.rawData = new rawData_type( r );
	m_refs.m_owns_raw_data = true;
      }


      inline void SetRawData( const rawData_type* r )
      {
	if ( r == m_refs.rawData )
	{
	  return;
	}
	if ( m_refs.m_owns_raw_data )
	{
	  delete m_refs.rawData;
	  m_refs.rawData = (rawData_type*)NULL;
	}
    
	m_refs.rawData = new rawData_type( *r );
	m_refs.m_owns_raw_data = true;
      }

	
      inline void SetRawData( rawData_type* r, bool allocate, bool owns )
      {
	if ( r == m_refs.rawData )
	{
	  return;
	}
	if ( m_refs.m_owns_raw_data )
	{
	  delete m_refs.rawData;
	  m_refs.rawData = (rawData_type*)NULL;
	}
	
	if ( allocate )
	{
	  owns = true;
	  m_refs.rawData = new rawData_type( *r );
	}
	else
	{
	  m_refs.rawData = r;
	}

	m_refs.m_owns_raw_data = owns;
      }

    public:
      inline rawData_type**
      AddressOfRawData( ) const
      {
	return &(const_cast< ref_type&>(m_refs).rawData );
      }


    protected:
      struct ref_type {

	bool			m_owns_raw_data;

	type_type		type;
	user_type		user;
	detectSim_type		detectSim;
	detectProc_type		detectProc;
	mutable history_type	history;
	rawData_type*		rawData;
	procData_type		procData;
	procData_type		strain;
	simData_type		simData;
	trigData_type		trigData;
	summaryData_type	summaryData;
	auxData_type		auxData;

	ref_type( )
	  : m_owns_raw_data( true ),
	    rawData( (rawData_type*)NULL )
	{
	  
	}

	ref_type( const ref_type& Source, const Common::FrameH::detector_copy_type Copy )
	  : m_owns_raw_data( true ),
	    rawData( (rawData_type*)NULL )
	{
	  rawData = (rawData_type*)NULL;
	  type = Source.type;
	  user = Source.user;
	  if ( Copy == Common::FrameH::SHALLOW_COPY_DETECTORS )
	  {
	    detectSim = Source.detectSim;
	    detectProc = Source.detectProc;
	  }
	  else
	  {
	    for ( const_detectSim_iterator d( Source.detectSim.begin( ) );
		  d != Source.detectSim.end( );
		  d++ )
	    {
	      detectSim.append( new FrDetector( *(*d) ),
				false, /* allocate */
				true /* own */  );
	    }
	    for ( const_detectProc_iterator d( Source.detectProc.begin( ) );
		  d != Source.detectProc.end( );
		  d++ )
	    {
	      detectProc.append( new FrDetector( *(*d) ),
				 false, /* allocate */
				 true /* own */ );
	    }
	  }
	  history = Source.history;
	  try
	  {
	    if ( Source.rawData != NULL)
	    {
	      if ( m_owns_raw_data )
	      {
		rawData = new rawData_type( *Source.rawData );
	      }
	      else
	      {
		rawData = Source.rawData;
	      }
	    }
	  }
	  catch( ... )
	  {
	    if ( m_owns_raw_data )
	    {
	      delete rawData;
	      rawData = (rawData_type*)NULL;
	    }
	    throw;
	  }
	  procData = Source.procData;
	  simData = Source.simData;
	  trigData = Source.trigData;
	  summaryData = Source.summaryData;
	  auxData = Source.auxData;
	}

	inline ~ref_type( )
	{
	  if ( rawData )
	  {
	    if ( m_owns_raw_data )
	    {	
	      delete rawData;
	    }
	    rawData = (rawData_type*)NULL;
	  }
	}

	inline
	cmn_streamsize_type Bytes( ) const
	{
	  return
	    ( PTR_STRUCT::Bytes( ) * MAX_REF );	// All the pointer elements
	}

	void
	readSubset( Common::IStream& Stream, INT_4U ElementMask )
	{
	  if ( ! ( ElementMask & Common::FrameH::DETECT_SIM ) )
	  {
	    Stream.RemoveResolver( &(detectSim), MAX_REF );
	  }
	  if ( ! ( ElementMask & Common::FrameH::DETECT_PROC ) )
	  {
	    Stream.RemoveResolver( &(detectProc), MAX_REF );
	  }
	  if ( ! ( ElementMask & Common::FrameH::HISTORY ) )
	  {
	    Stream.RemoveResolver( &(history), MAX_REF );
	  }
	  Stream.RemoveResolver( &(type), MAX_REF );
	  Stream.RemoveResolver( &(user), MAX_REF );
	  if ( ! ( ElementMask & Common::FrameH::RAW_DATA ) )
	  {
	    Stream.RemoveResolver( &(rawData), MAX_REF );
	  }
	  Stream.RemoveResolver( &(procData), MAX_REF );
	  Stream.RemoveResolver( &(simData), MAX_REF );
	  Stream.RemoveResolver( &(trigData), MAX_REF );
	  Stream.RemoveResolver( &(summaryData), MAX_REF );
	  if ( ! ( ElementMask & Common::FrameH::AUX_DATA ) )
	  {
	    Stream.RemoveResolver( &(auxData), MAX_REF );
	  }
	}

	inline bool
	operator==( const ref_type& RHS ) const
	{
	  return ( ( this == &RHS )
		   || ( ( type == RHS.type )
			&& ( user == RHS.user )
			&& ( detectSim == RHS.detectSim )
			&& ( detectProc == RHS.detectProc )
			&& ( history == RHS.history )
			&& ( ( rawData == RHS.rawData )
			     || ( ( rawData != (rawData_type*)NULL )
				  && ( RHS.rawData != (rawData_type*)NULL )
				  && ( *rawData == *(RHS.rawData) ) ) )
			&& ( procData == RHS.procData )
			&& ( simData == RHS.simData )
			&& ( trigData == RHS.trigData )
			&& ( summaryData == RHS.summaryData )
			&& ( auxData == RHS.auxData )
			) );
	}

	inline void
	operator( )( Common::IStream& Stream )
	{
	  void* rd = &rawData;

	  Stream
	    >> type
	    >> user
	    >> detectSim
	    >> detectProc
	    >> history
	    >> static_cast< Common::FrameSpec::Object** >( rd )
	    >> procData
	    >> strain
	    >> simData
	    >> trigData
	    >> summaryData
	    >> auxData
	    ;
	}
      
	inline void
	operator( )( Common::OStream& Stream ) const
	{
	  const void* raw_data = rawData;

	  Stream
	    << type
	    << user
	    << detectSim
	    << detectProc
	    << history
	    << static_cast< const Common::FrameSpec::Object* >( raw_data )
	    << procData
	    << strain
	    << simData
	    << trigData
	    << summaryData
	    << auxData
	    ;
	}
      
      };

      ref_type	m_refs;

      static inline void
      refDescription( Common::Description& Desc )
      {
	Desc( FrSE( "type", PTR_STRUCT::Desc( FrVect::StructName( ) ),
		    "" ) );
	Desc( FrSE( "user", PTR_STRUCT::Desc( FrVect::StructName( ) ),
		    "" ) );
	Desc( FrSE( "detectSim", PTR_STRUCT::Desc( FrDetector::StructName( ) ),
		    "" ) );
	Desc( FrSE( "detectProc", PTR_STRUCT::Desc( FrDetector::StructName( ) ),
		    "" ) );
	Desc( FrSE( "history", PTR_STRUCT::Desc(FrHistory::StructName( ) ),
		    "" ) );
	Desc( FrSE( "rawData",  PTR_STRUCT::Desc( rawData_type::StructName( ) ),
		    "" ) );
	Desc( FrSE( "procData", PTR_STRUCT::Desc( FrProcData::StructName( ) ),
		    "" ) );
	Desc( FrSE( "strain", PTR_STRUCT::Desc( FrProcData::StructName( ) ),
		    "" ) );
	Desc( FrSE( "simData", PTR_STRUCT::Desc( FrSimData::StructName( ) ),
		    "" ) );
	Desc( FrSE( "trigData", PTR_STRUCT::Desc( FrTrigData::StructName( ) ),
		    "" ) );
	Desc( FrSE( "summaryData", PTR_STRUCT::Desc( FrSummary::StructName( ) ),
		    "" ) );
	Desc( FrSE( "auxData", PTR_STRUCT::Desc( FrVect::StructName( ) ),
		    "" ) );
      }

    };

    typedef FrameHRefsT< FrDetector, FrTrigData, FrHistory,
			 FrProcData, FrRawData, FrSimData,
			 FrSummary, FrVect >
    FrameHRefs;

    //===================================================================
    //===================================================================
    class FrameH
      : public Common::FrameH,
        public FrameHStorage,
	public FrameHRefs
    {
    public:
      using Common::FrameSpec::Object::Create;

      //: Default constructor
      FrameH( );
      FrameH( const FrameH& frame,
	      detector_copy_type Copy = SHALLOW_COPY_DETECTORS );
      FrameH( const std::string& name,
	      run_type run,
	      INT_4U frame,
	      const GPSTime& time,
	      INT_2U uLeapS,
	      INT_4S LocalTime,
	      const REAL_8 dt,
	      INT_4U dqual = 0 );
      FrameH( const FrameHStorage& Source );
      ~FrameH( );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrameH* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      //: Write the structure to the stream
      virtual void Write( Common::OStream& Stream ) const;

      /// \brief comparison operator
      bool operator==( const FrameH& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      // Interface routines
      //-----------------------------------------------------------------
      virtual void readSubset( Common::IStream& Stream,
			       INT_4U ElementMask );

    private:
      FrameH( Common::IStream& Stream );

      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrameH::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Common::PromoteObject( DATA_FORMAT_VERSION, Source, Obj );
    }

    inline const char* const FrameH::
    StructName( )
    {
      static const char* const name = "FrameH";
      return name;
    }

    inline bool FrameH::
    operator==( const FrameH& RHS ) const
    {
      return ( ( m_data == RHS.m_data )
	       && ( m_refs == RHS.m_refs ) );
    }



  } // namespace - Version_3
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_3_FrameH_HH */
