#ifndef FrameCPP_VERSION_6_FrSimEvent_HH
#define FrameCPP_VERSION_6_FrSimEvent_HH

#include <string>
#include <vector>

#include "framecpp/FrameCPP.hh"

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version4/FrSimEvent.hh"

#include "framecpp/Version6/FrameSpec.hh"
#include "framecpp/Version6/GPSTime.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    // FrSimEventStorage
    //===================================================================
    class FrSimEventStorage
    {
    public:
      typedef INT_2U				nParam_type;
      typedef std::pair< STRING, REAL_4 >	Param_type;
      typedef std::vector< Param_type >		ParamList_type;

      FrSimEventStorage( );
      FrSimEventStorage( const FrSimEventStorage& Source );
      FrSimEventStorage( const Previous::FrSimEventStorage& Source );

      const std::string& GetName() const { return m_data.name; }
      const std::string& GetComment( ) const { return m_data.comment; }
      const std::string& GetInputs( ) const { return m_data.inputs; }
      const GPSTime& GetGTime( ) const { return m_data.GTime; }
      REAL_4 GetTimeBefore( ) const { return m_data.timeBefore; }
      REAL_4 GetTimeAfter( ) const { return m_data.timeAfter; }
      REAL_4 GetAmplitude( ) const { return m_data.amplitude; }
      const ParamList_type& GetParam( ) const { return m_data.Params; }
      ParamList_type& GetParam( ) { return m_data.Params; }

    protected:
      struct data_type {
	STRING	name;
	STRING	comment;
	STRING	inputs;
	GPSTime	GTime;
	REAL_4	timeBefore;
	REAL_4	timeAfter;
	REAL_4	amplitude;
	ParamList_type	Params;

	data_type( );
	data_type( const data_type& Source );

	cmn_streamsize_type Bytes( ) const;

	bool operator==( const data_type& RHS ) const;

	void operator( )( Common::IStream& Stream );
	void operator( )( Common::OStream& Stream ) const;
      };

      data_type	m_data;
    };

    inline FrSimEventStorage::data_type::
    data_type( )
    {
    }

    inline FrSimEventStorage::data_type::
    data_type( const data_type& Source )
      : name( Source.name ),
	comment( Source.comment ),
	inputs( Source.inputs ),
	GTime( Source.GTime ),
	timeBefore( Source.timeBefore ),
	timeAfter( Source.timeAfter ),
	amplitude( Source.amplitude ),
	Params( Source.Params )
    {
    }

    inline bool FrSimEventStorage::data_type::
    operator==( const data_type& RHS ) const
    {
      return ( ( &RHS == this )
	       || ( ( name == RHS.name )
		    && ( comment == RHS.comment )
		    && ( inputs == RHS.inputs )
		    && ( GTime == RHS.GTime )
		    && ( timeBefore == RHS.timeBefore )
		    && ( timeAfter == RHS.timeAfter )
		    && ( amplitude == RHS.amplitude )
		    && ( Params == RHS.Params )
		) );
    }


    inline FrSimEventStorage::
    FrSimEventStorage( )
    {
    }

    inline FrSimEventStorage::
    FrSimEventStorage( const FrSimEventStorage& Source )
      : m_data( Source.m_data )
    {
    }

    inline FrSimEventStorage::
    FrSimEventStorage( const Previous::FrSimEventStorage& Source )
    {
      m_data.name = Source.GetName( );
      m_data.comment = Source.GetComment( );
      m_data.inputs = Source.GetInputs( );
      m_data.GTime = Source.GetGTime( );
      m_data.timeBefore = Source.GetTimeBefore( );
      m_data.timeAfter = Source.GetTimeAfter( );
      m_data.amplitude = Source.GetAmplitude( );
    }

    //===================================================================
    // FrSimEventRefsT
    //===================================================================

    typedef Version_4::FrSimEventRefsT< FrVect, FrTable> FrSimEventRefs;

    //===================================================================
    // FrSimEvent
    //===================================================================
    class FrSimEvent
      : public Common::FrameSpec::Object,
	public FrSimEventStorage,
	public FrSimEventRefs,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrSimEvent( );
      FrSimEvent( const FrSimEvent& eventData );
      FrSimEvent( const std::string& name, const std::string& comment,
		  const std::string& inputs,
		  const GPSTime& time,
		  const REAL_4 timeBefore,
		  const REAL_4 timeAfter,
		  const REAL_4 amplitude,
		  const ParamList_type& parameters );
      FrSimEvent( const Previous::FrSimEvent& Source,
	       Common::IStream* Stream );

      const std::string& GetNameSlow( ) const;

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrSimEvent* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      //: Merge with another FrSimEvent
      FrSimEvent& Merge( const FrSimEvent& RHS );

      /// \brief comparison operator
      bool operator==( const FrSimEvent& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrSimEvent( Common::IStream& Stream );
    };

    inline FrameCPP::cmn_streamsize_type FrSimEvent::
    Bytes( const Common::StreamBase& Stream ) const
    {
      return
	m_data.Bytes( )
	+ m_refs.Bytes( Stream )
	+ Stream.PtrStructBytes( )	// next
	;
    }

    inline std::auto_ptr< Common::FrameSpec::Object > FrSimEvent::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrSimEvent, FrSimEvent >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrSimEvent::
    StructName( )
    {
      static const char* const name = "FrSimEvent";
      return name;
    }

    inline bool FrSimEvent::
    operator==( const FrSimEvent& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && (m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrSimEvent_HH */
