#ifndef FrameCPP_VERSION_7_FrSimData_HH
#define FrameCPP_VERSION_7_FrSimData_HH

#include <vector>

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version6/FrSimData.hh"

#include "framecpp/Version7/FrameSpec.hh"
#include "framecpp/Version7/STRING.hh"

#include "framecpp/Version7/FrHistory.hh"
#include "framecpp/Version7/FrTable.hh"
#include "framecpp/Version7/FrVect.hh"

namespace FrameCPP
{
  namespace Version_7
  {
    //===================================================================
    //===================================================================
    class FrSimDataStorage
    {
    public:
      static const REAL_8 DEFAULT_TIME_OFFSET;

      FrSimDataStorage( );
      FrSimDataStorage( const FrSimDataStorage& Source );
      FrSimDataStorage( const std::string& Name,
			const std::string& Comment,
			const REAL_8 SampleRate, 
			const REAL_8 FShift,
			const REAL_4 Phase,
			const REAL_8 TimeOffset );
      FrSimDataStorage( const Previous::FrSimDataStorage& Source );

      const std::string& GetName( ) const;
      const std::string& GetComment( ) const;
      REAL_8 GetSampleRate( ) const;
      REAL_8 GetTimeOffset( ) const;
      REAL_8 GetFShift( ) const;
      REAL_4 GetPhase( ) const;
   
    protected:
      struct data_type {
	STRING		name;
	STRING		comment;
	REAL_8		sampleRate;
	REAL_8		timeOffset;
	REAL_8		fShift;
	REAL_4		phase;

	inline FrameCPP::cmn_streamsize_type
	Bytes( const Common::StreamBase& Stream ) const
	{
	  return
	    name.Bytes( )
	    + comment.Bytes( )
	    + sizeof( sampleRate )
	    + sizeof( timeOffset )
	    + sizeof( fShift )
	    + sizeof( phase )
	    ;
	}

	inline bool
	operator==( const data_type& RHS ) const
	{
	  return ( ( this == &RHS )
		   || ( ( name == RHS.name )
			&& ( comment == RHS.comment )
			&& ( sampleRate == RHS.sampleRate )
			&& ( timeOffset == RHS.timeOffset )
			&& ( fShift == RHS.fShift )
			&& ( phase == RHS.phase )
			) );
	}

	void operator( )( Common::IStream& Stream );
	void operator( )( Common::OStream& Stream ) const;
      };

      data_type m_data;
    };

    inline const std::string& FrSimDataStorage::
    GetName() const
    {
      return m_data.name;
    }

    inline const std::string& FrSimDataStorage::
    GetComment() const
    {
      return m_data.comment;
    }

    inline REAL_8 FrSimDataStorage::
    GetSampleRate( ) const
    {
      return m_data.sampleRate;
    }

    inline REAL_8 FrSimDataStorage::
    GetTimeOffset( ) const
    {
      return m_data.timeOffset;
    }

    inline REAL_8 FrSimDataStorage::
    GetFShift( ) const
    {
      return m_data.fShift;
    }

    inline REAL_4 FrSimDataStorage::
    GetPhase( ) const
    {
      return m_data.phase;
    }

    //===================================================================
    // FrSimDataRefs
    //===================================================================

    typedef Version_4::FrSimDataRefs< FrVect, FrTable> FrSimDataRefs;

    //===================================================================
    //===================================================================
    class FrSimData
      : public Common::FrameSpec::Object,
	public FrSimDataStorage,
	public FrSimDataRefs,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrSimData( );
      FrSimData( const FrSimData& SimData );
      FrSimData( const std::string& Name,
		 const std::string& Comment,
		 const REAL_8 SampleRate, 
		 const REAL_8 FShift,
		 const REAL_4 Phase,
		 const REAL_8 TimeOffset = DEFAULT_TIME_OFFSET );
      FrSimData( Previous::FrSimData& Source,
		 Common::IStream* Stream );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrSimData* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      /// \brief Retrive the name
      const std::string& GetNameSlow() const;
      
      //: Merge with another FrAdcData
      FrSimData& Merge( const FrSimData& RHS );

      /// \brief comparison operator
      bool operator==( const FrSimData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrSimData( IStream& Stream );
    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrSimData::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrSimData, FrSimData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrSimData::
    StructName( )
    {
      static const char* const name = "FrSimData";
      return name;
    }

    inline bool FrSimData::
    operator==( const FrSimData& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && ( m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_7
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_7_FrSimData_HH */
